# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: update_metadata.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15update_metadata.proto\x12\x16\x63hromeos_update_engine\"\xe0\x02\n\x10InstallOperation\x12;\n\x04type\x18\x01 \x02(\x0e\x32-.chromeos_update_engine.InstallOperation.Type\x12\x13\n\x0b\x64\x61ta_offset\x18\x02 \x01(\r\x12\x13\n\x0b\x64\x61ta_length\x18\x03 \x01(\r\x12\x33\n\x0bsrc_extents\x18\x04 \x03(\x0b\x32\x1e.chromeos_update_engine.Extent\x12\x12\n\nsrc_length\x18\x05 \x01(\x04\x12\x33\n\x0b\x64st_extents\x18\x06 \x03(\x0b\x32\x1e.chromeos_update_engine.Extent\x12\x12\n\ndst_length\x18\x07 \x01(\x04\x12\x18\n\x10\x64\x61ta_sha256_hash\x18\x08 \x01(\x0c\"9\n\x04Type\x12\x0b\n\x07REPLACE\x10\x00\x12\x0e\n\nREPLACE_BZ\x10\x01\x12\x08\n\x04MOVE\x10\x02\x12\n\n\x06\x42SDIFF\x10\x03\"1\n\x06\x45xtent\x12\x13\n\x0bstart_block\x18\x01 \x01(\x04\x12\x12\n\nnum_blocks\x18\x02 \x01(\x04\"z\n\nSignatures\x12@\n\nsignatures\x18\x01 \x03(\x0b\x32,.chromeos_update_engine.Signatures.Signature\x1a*\n\tSignature\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\")\n\x0bInstallInfo\x12\x0c\n\x04size\x18\x01 \x01(\x04\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\"\x8f\x02\n\x10InstallProcedure\x12;\n\x04type\x18\x01 \x01(\x0e\x32-.chromeos_update_engine.InstallProcedure.Type\x12<\n\noperations\x18\x02 \x03(\x0b\x32(.chromeos_update_engine.InstallOperation\x12\x35\n\x08old_info\x18\x03 \x01(\x0b\x32#.chromeos_update_engine.InstallInfo\x12\x35\n\x08new_info\x18\x04 \x01(\x0b\x32#.chromeos_update_engine.InstallInfo\"\x12\n\x04Type\x12\n\n\x06KERNEL\x10\x00\"\xaf\x03\n\x14\x44\x65ltaArchiveManifest\x12\x46\n\x14partition_operations\x18\x01 \x03(\x0b\x32(.chromeos_update_engine.InstallOperation\x12\x41\n\x0fnoop_operations\x18\x02 \x03(\x0b\x32(.chromeos_update_engine.InstallOperation\x12\x18\n\nblock_size\x18\x03 \x01(\r:\x04\x34\x30\x39\x36\x12\x19\n\x11signatures_offset\x18\x04 \x01(\x04\x12\x17\n\x0fsignatures_size\x18\x05 \x01(\x04\x12?\n\x12old_partition_info\x18\x08 \x01(\x0b\x32#.chromeos_update_engine.InstallInfo\x12?\n\x12new_partition_info\x18\t \x01(\x0b\x32#.chromeos_update_engine.InstallInfo\x12<\n\nprocedures\x18\n \x03(\x0b\x32(.chromeos_update_engine.InstallProcedure')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'update_metadata_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_INSTALLOPERATION']._serialized_start=50
  _globals['_INSTALLOPERATION']._serialized_end=402
  _globals['_INSTALLOPERATION_TYPE']._serialized_start=345
  _globals['_INSTALLOPERATION_TYPE']._serialized_end=402
  _globals['_EXTENT']._serialized_start=404
  _globals['_EXTENT']._serialized_end=453
  _globals['_SIGNATURES']._serialized_start=455
  _globals['_SIGNATURES']._serialized_end=577
  _globals['_SIGNATURES_SIGNATURE']._serialized_start=535
  _globals['_SIGNATURES_SIGNATURE']._serialized_end=577
  _globals['_INSTALLINFO']._serialized_start=579
  _globals['_INSTALLINFO']._serialized_end=620
  _globals['_INSTALLPROCEDURE']._serialized_start=623
  _globals['_INSTALLPROCEDURE']._serialized_end=894
  _globals['_INSTALLPROCEDURE_TYPE']._serialized_start=876
  _globals['_INSTALLPROCEDURE_TYPE']._serialized_end=894
  _globals['_DELTAARCHIVEMANIFEST']._serialized_start=897
  _globals['_DELTAARCHIVEMANIFEST']._serialized_end=1328
# @@protoc_insertion_point(module_scope)
